@@ -63,6 +63,9 @@ echo "ZSTD: $ZSTD"
 OS=$(uname -s | tr '[:upper:]' '[:lower:]')
 echo "OS: $OS"
 
+# Assume we are running in the mongo-c-driver source root directory
+MONGOC_DIR="$PWD"
+
 # Since zstd inconsitently installed on macos-1014.
 # Remove this check in CDRIVER-3483.
 if [ "darwin" = "$OS" ]; then
@@ -230,11 +233,13 @@ pkg-config --modversion libssl || true
 
 if [ "$COMPILE_LIBMONGOCRYPT" = "ON" ]; then
    # Build libmongocrypt, using the previously fetched installed source.
-   git clone https://github.com/mongodb/libmongocrypt --branch 1.5.0
+   git clone https://github.com/mongodb/libmongocrypt --branch 1.5.2
 
    mkdir libmongocrypt/cmake-build
    cd libmongocrypt/cmake-build
-   $CMAKE -DENABLE_SHARED_BSON=ON -DCMAKE_BUILD_TYPE="Debug" -DCMAKE_INSTALL_PREFIX="$INSTALL_DIR" -DCMAKE_PREFIX_PATH="$CMAKE_PREFIX_PATH" ../
+   $CMAKE -DENABLE_SHARED_BSON=ON -DCMAKE_BUILD_TYPE="Debug" \
+      -DMONGOCRYPT_MONGOC_DIR="$MONGOC_DIR" -DCMAKE_INSTALL_PREFIX="$INSTALL_DIR" \
+      -DCMAKE_PREFIX_PATH="$CMAKE_PREFIX_PATH" -DBUILD_TESTING=OFF ../
    make install
    cd ../../
    else
