@@ -18,7 +18,7 @@ export TSAN_OPTIONS="suppressions=./.tsan-suppressions"
 echo "COMPRESSORS='${COMPRESSORS}' CC='${CC}' AUTH=${AUTH} SSL=${SSL} URI=${URI} IPV4_ONLY=${IPV4_ONLY} VALGRIND=${VALGRIND} MONGOC_TEST_URI=${MONGOC_TEST_URI}"
 
 [ -z "$MARCH" ] && MARCH=$(uname -m | tr '[:upper:]' '[:lower:]')
-TEST_ARGS="-d -F test-results.json"
+TEST_ARGS="-d -F test-results.json --skip-tests .evergreen/skip-tests.txt"
 
 if [ "$COMPRESSORS" != "nocompressors" ]; then
    export MONGOC_TEST_COMPRESSORS="$COMPRESSORS"
@@ -93,7 +93,7 @@ export MONGOC_TEST_MONITORING_VERBOSE=on
 if [ "$CLIENT_SIDE_ENCRYPTION" = "on" ]; then
    echo "Waiting for mock KMS servers to start..."
    wait_for_kms_server() {
-      for i in $(seq 60); do
+      for i in $(seq 300); do
          # Exit code 7: "Failed to connect to host".
          if curl -s "localhost:$1"; test $? -ne 7; then
             return 0
@@ -110,6 +110,11 @@ if [ "$CLIENT_SIDE_ENCRYPTION" = "on" ]; then
    wait_for_kms_server 8002
    wait_for_kms_server 5698
    echo "Waiting for mock KMS servers to start... done."
+   if ! test -d /cygdrive/c; then
+      # We have trouble with this test on Windows. only set cryptSharedLibPath on other platforms
+      export MONGOC_TEST_CRYPT_SHARED_LIB_PATH="$(find . -wholename '*src/libmongoc/mongo_crypt_v1.*' -and -regex '.*\(.dll\|.dylib\|.so\)' | head -n1)"
+      echo "Setting env cryptSharedLibPath: [$MONGOC_TEST_CRYPT_SHARED_LIB_PATH]"
+   fi
 fi
 
 if [ "$LOADBALANCED" != "noloadbalanced" ]; then
@@ -117,7 +122,7 @@ if [ "$LOADBALANCED" != "noloadbalanced" ]; then
       echo "SINGLE_MONGOS_LB_URI and MULTI_MONGOS_LB_URI environment variables required."
       exit 1
    fi
-   
+
    export MONGOC_TEST_LOADBALANCED=ON
 
    TEST_ARGS="$TEST_ARGS -l /unified/*"
