@@ -134,20 +134,35 @@ mongoc_async_cmd_run (mongoc_async_cmd_t *acmd)
 }
 
 void
-_mongoc_async_cmd_init_send (mongoc_async_cmd_t *acmd, const char *dbname)
+_mongoc_async_cmd_init_send (const mongoc_opcode_t cmd_opcode,
+                             mongoc_async_cmd_t *acmd,
+                             const char *dbname)
 {
-   acmd->ns = bson_strdup_printf ("%s.$cmd", dbname);
-
    acmd->rpc.header.msg_len = 0;
    acmd->rpc.header.request_id = ++acmd->async->request_id;
    acmd->rpc.header.response_to = 0;
-   acmd->rpc.header.opcode = MONGOC_OPCODE_QUERY;
-   acmd->rpc.query.flags = MONGOC_QUERY_SECONDARY_OK;
-   acmd->rpc.query.collection = acmd->ns;
-   acmd->rpc.query.skip = 0;
-   acmd->rpc.query.n_return = -1;
-   acmd->rpc.query.query = bson_get_data (&acmd->cmd);
-   acmd->rpc.query.fields = NULL;
+
+   if (MONGOC_OPCODE_QUERY == cmd_opcode) {
+      acmd->ns = bson_strdup_printf ("%s.$cmd", dbname);
+      acmd->rpc.header.opcode = MONGOC_OPCODE_QUERY;
+      acmd->rpc.query.flags = MONGOC_QUERY_SECONDARY_OK;
+      acmd->rpc.query.collection = acmd->ns;
+      acmd->rpc.query.skip = 0;
+      acmd->rpc.query.n_return = -1;
+      acmd->rpc.query.query = bson_get_data (&acmd->cmd);
+      acmd->rpc.query.fields = NULL;
+   }
+
+   if (MONGOC_OPCODE_MSG == cmd_opcode) {
+      acmd->rpc.header.opcode = MONGOC_OPCODE_MSG;
+
+      acmd->rpc.msg.msg_len = 0;
+      acmd->rpc.msg.flags = 0;
+      acmd->rpc.msg.n_sections = 1;
+      acmd->rpc.msg.sections[0].payload_type = 0;
+      acmd->rpc.msg.sections[0].payload.bson_document =
+         bson_get_data (&acmd->cmd);
+   }
 
    /* This will always be hello, which are not allowed to be compressed */
    _mongoc_rpc_gather (&acmd->rpc, &acmd->array);
@@ -182,6 +197,7 @@ mongoc_async_cmd_new (mongoc_async_t *async,
                       void *setup_ctx,
                       const char *dbname,
                       const bson_t *cmd,
+                      const mongoc_opcode_t cmd_opcode, /* OP_QUERY or OP_MSG */
                       mongoc_async_cmd_cb_t cb,
                       void *cb_data,
                       int64_t timeout_msec)
@@ -205,10 +221,15 @@ mongoc_async_cmd_new (mongoc_async_t *async,
    acmd->connect_started = bson_get_monotonic_time ();
    bson_copy_to (cmd, &acmd->cmd);
 
+   if (MONGOC_OPCODE_MSG == cmd_opcode) {
+      /* If we're sending an OPCODE_MSG, we need to add the "db" field: */
+      bson_append_utf8 (&acmd->cmd, "$db", 3, "admin", 5);
+   }
+
    _mongoc_array_init (&acmd->array, sizeof (mongoc_iovec_t));
    _mongoc_buffer_init (&acmd->buffer, NULL, 0, NULL, NULL);
 
-   _mongoc_async_cmd_init_send (acmd, dbname);
+   _mongoc_async_cmd_init_send (cmd_opcode, acmd, dbname);
 
    _mongoc_async_cmd_state_start (acmd, is_setup_done);
 
