@@ -54,6 +54,8 @@ mongoc_bulk_operation_new (bool ordered)
    bulk->flags.ordered = ordered;
    bulk->server_id = 0;
 
+   bson_init (&bulk->let);
+
    _mongoc_array_init (&bulk->commands, sizeof (mongoc_write_command_t));
    _mongoc_write_result_init (&bulk->result);
 
@@ -71,8 +73,8 @@ _mongoc_bulk_operation_new (
 {
    mongoc_bulk_operation_t *bulk;
 
-   BSON_ASSERT (client);
-   BSON_ASSERT (collection);
+   BSON_ASSERT_PARAM (client);
+   BSON_ASSERT_PARAM (collection);
 
    bulk = mongoc_bulk_operation_new (flags.ordered);
    bulk->client = client;
@@ -102,6 +104,8 @@ mongoc_bulk_operation_destroy (mongoc_bulk_operation_t *bulk) /* IN */
 
       bson_free (bulk->database);
       bson_free (bulk->collection);
+      bson_value_destroy (&bulk->comment);
+      bson_destroy (&bulk->let);
       mongoc_write_concern_destroy (bulk->write_concern);
       _mongoc_array_destroy (&bulk->commands);
 
@@ -135,7 +139,7 @@ mongoc_bulk_operation_destroy (mongoc_bulk_operation_t *bulk) /* IN */
    } while (0)
 
 
-bool
+static bool
 _mongoc_bulk_operation_remove_with_opts (
    mongoc_bulk_operation_t *bulk,
    const bson_t *selector,
@@ -145,6 +149,7 @@ _mongoc_bulk_operation_remove_with_opts (
 {
    mongoc_write_command_t command = {0};
    mongoc_write_command_t *last;
+   bson_t cmd_opts = BSON_INITIALIZER;
    bson_t opts;
    bool has_collation;
    bool ret = false;
@@ -152,8 +157,8 @@ _mongoc_bulk_operation_remove_with_opts (
 
    ENTRY;
 
-   BSON_ASSERT (bulk);
-   BSON_ASSERT (selector);
+   BSON_ASSERT_PARAM (bulk);
+   BSON_ASSERT_PARAM (selector);
 
    bson_init (&opts);
 
@@ -193,8 +198,16 @@ _mongoc_bulk_operation_remove_with_opts (
       }
    }
 
+   if (bulk->comment.value_type != BSON_TYPE_EOD) {
+      bson_append_value (&cmd_opts, "comment", 7, &bulk->comment);
+   }
+
+   if (!bson_empty (&bulk->let)) {
+      bson_append_document (&cmd_opts, "let", 3, &bulk->let);
+   }
+
    _mongoc_write_command_init_delete (
-      &command, selector, NULL, &opts, bulk->flags, bulk->operation_id);
+      &command, selector, &cmd_opts, &opts, bulk->flags, bulk->operation_id);
 
    command.flags.has_collation = has_collation;
    command.flags.has_delete_hint = has_delete_hint;
@@ -204,6 +217,7 @@ _mongoc_bulk_operation_remove_with_opts (
    ret = true;
 
 done:
+   bson_destroy (&cmd_opts);
    bson_destroy (&opts);
    RETURN (ret);
 }
@@ -336,8 +350,8 @@ mongoc_bulk_operation_insert (mongoc_bulk_operation_t *bulk,
 {
    ENTRY;
 
-   BSON_ASSERT (bulk);
-   BSON_ASSERT (document);
+   BSON_ASSERT_PARAM (bulk);
+   BSON_ASSERT_PARAM (document);
 
    if (!mongoc_bulk_operation_insert_with_opts (
           bulk, document, NULL /* opts */, &bulk->result.error)) {
@@ -356,12 +370,13 @@ mongoc_bulk_operation_insert_with_opts (mongoc_bulk_operation_t *bulk,
    mongoc_bulk_insert_opts_t insert_opts;
    mongoc_write_command_t command = {0};
    mongoc_write_command_t *last;
+   bson_t cmd_opts = BSON_INITIALIZER;
    bool ret = false;
 
    ENTRY;
 
-   BSON_ASSERT (bulk);
-   BSON_ASSERT (document);
+   BSON_ASSERT_PARAM (bulk);
+   BSON_ASSERT_PARAM (document);
 
    BULK_RETURN_IF_PRIOR_ERROR;
 
@@ -374,6 +389,9 @@ mongoc_bulk_operation_insert_with_opts (mongoc_bulk_operation_t *bulk,
       GOTO (done);
    }
 
+   /* Note: mongoc_bulk_insert_opts_t specifies allow_extra=False, so there is
+    * no reason to concatenate cmd_opts with &insert_opts.extra. */
+
    if (bulk->commands.len) {
       last = &_mongoc_array_index (
          &bulk->commands, mongoc_write_command_t, bulk->commands.len - 1);
@@ -385,8 +403,12 @@ mongoc_bulk_operation_insert_with_opts (mongoc_bulk_operation_t *bulk,
       }
    }
 
+   if (bulk->comment.value_type != BSON_TYPE_EOD) {
+      bson_append_value (&cmd_opts, "comment", 7, &bulk->comment);
+   }
+
    _mongoc_write_command_init_insert (
-      &command, document, &insert_opts.extra, bulk->flags, bulk->operation_id);
+      &command, document, &cmd_opts, bulk->flags, bulk->operation_id);
 
    _mongoc_array_append_val (&bulk->commands, command);
 
@@ -394,6 +416,8 @@ mongoc_bulk_operation_insert_with_opts (mongoc_bulk_operation_t *bulk,
 
 done:
    _mongoc_bulk_insert_opts_cleanup (&insert_opts);
+   bson_destroy (&cmd_opts);
+
    RETURN (ret);
 }
 
@@ -408,6 +432,7 @@ _mongoc_bulk_operation_update_append (
 {
    mongoc_write_command_t command = {0};
    mongoc_write_command_t *last;
+   bson_t cmd_opts = BSON_INITIALIZER;
    bson_t opts;
    bool has_collation;
    bool has_array_filters;
@@ -445,13 +470,25 @@ _mongoc_bulk_operation_update_append (
          last->flags.has_update_hint |= has_update_hint;
          last->flags.has_multi_write |= update_opts->multi;
          _mongoc_write_command_update_append (last, selector, document, &opts);
-         bson_destroy (&opts);
-         return;
+         GOTO (done);
       }
    }
 
-   _mongoc_write_command_init_update (
-      &command, selector, document, &opts, bulk->flags, bulk->operation_id);
+   if (bulk->comment.value_type != BSON_TYPE_EOD) {
+      bson_append_value (&cmd_opts, "comment", 7, &bulk->comment);
+   }
+
+   if (!bson_empty (&bulk->let)) {
+      bson_append_document (&cmd_opts, "let", 3, &bulk->let);
+   }
+
+   _mongoc_write_command_init_update (&command,
+                                      selector,
+                                      document,
+                                      &cmd_opts,
+                                      &opts,
+                                      bulk->flags,
+                                      bulk->operation_id);
 
    command.flags.has_array_filters = has_array_filters;
    command.flags.has_collation = has_collation;
@@ -459,6 +496,9 @@ _mongoc_bulk_operation_update_append (
    command.flags.has_multi_write = update_opts->multi;
 
    _mongoc_array_append_val (&bulk->commands, command);
+
+done:
+   bson_destroy (&cmd_opts);
    bson_destroy (&opts);
 }
 
@@ -475,9 +515,9 @@ _mongoc_bulk_operation_update_with_opts (
 {
    ENTRY;
 
-   BSON_ASSERT (bulk);
-   BSON_ASSERT (selector);
-   BSON_ASSERT (document);
+   BSON_ASSERT_PARAM (bulk);
+   BSON_ASSERT_PARAM (selector);
+   BSON_ASSERT_PARAM (document);
 
    if (!_mongoc_validate_update (document, update_opts->validate, error)) {
       RETURN (false);
@@ -642,9 +682,9 @@ mongoc_bulk_operation_replace_one_with_opts (mongoc_bulk_operation_t *bulk,
 
    ENTRY;
 
-   BSON_ASSERT (bulk);
-   BSON_ASSERT (selector);
-   BSON_ASSERT (document);
+   BSON_ASSERT_PARAM (bulk);
+   BSON_ASSERT_PARAM (selector);
+   BSON_ASSERT_PARAM (document);
 
    BULK_RETURN_IF_PRIOR_ERROR;
 
@@ -714,7 +754,7 @@ mongoc_bulk_operation_execute (mongoc_bulk_operation_t *bulk, /* IN */
 
    ENTRY;
 
-   BSON_ASSERT (bulk);
+   BSON_ASSERT_PARAM (bulk);
 
    if (!bulk->client) {
       bson_set_error (error,
@@ -836,7 +876,7 @@ void
 mongoc_bulk_operation_set_write_concern (
    mongoc_bulk_operation_t *bulk, const mongoc_write_concern_t *write_concern)
 {
-   BSON_ASSERT (bulk);
+   BSON_ASSERT_PARAM (bulk);
 
    if (bulk->write_concern) {
       mongoc_write_concern_destroy (bulk->write_concern);
@@ -852,7 +892,7 @@ mongoc_bulk_operation_set_write_concern (
 const mongoc_write_concern_t *
 mongoc_bulk_operation_get_write_concern (const mongoc_bulk_operation_t *bulk)
 {
-   BSON_ASSERT (bulk);
+   BSON_ASSERT_PARAM (bulk);
 
    return bulk->write_concern;
 }
@@ -862,7 +902,7 @@ void
 mongoc_bulk_operation_set_database (mongoc_bulk_operation_t *bulk,
                                     const char *database)
 {
-   BSON_ASSERT (bulk);
+   BSON_ASSERT_PARAM (bulk);
 
    if (bulk->database) {
       bson_free (bulk->database);
@@ -876,7 +916,7 @@ void
 mongoc_bulk_operation_set_collection (mongoc_bulk_operation_t *bulk,
                                       const char *collection)
 {
-   BSON_ASSERT (bulk);
+   BSON_ASSERT_PARAM (bulk);
 
    if (bulk->collection) {
       bson_free (bulk->collection);
@@ -889,8 +929,8 @@ mongoc_bulk_operation_set_collection (mongoc_bulk_operation_t *bulk,
 void
 mongoc_bulk_operation_set_client (mongoc_bulk_operation_t *bulk, void *client)
 {
-   BSON_ASSERT (bulk);
-   BSON_ASSERT (client);
+   BSON_ASSERT_PARAM (bulk);
+   BSON_ASSERT_PARAM (client);
 
    if (bulk->session) {
       BSON_ASSERT (bulk->session->client == client);
@@ -912,8 +952,8 @@ mongoc_bulk_operation_set_client_session (
    mongoc_bulk_operation_t *bulk,
    struct _mongoc_client_session_t *client_session)
 {
-   BSON_ASSERT (bulk);
-   BSON_ASSERT (client_session);
+   BSON_ASSERT_PARAM (bulk);
+   BSON_ASSERT_PARAM (client_session);
 
    if (bulk->client) {
       BSON_ASSERT (bulk->client == client_session->client);
@@ -926,7 +966,7 @@ mongoc_bulk_operation_set_client_session (
 uint32_t
 mongoc_bulk_operation_get_hint (const mongoc_bulk_operation_t *bulk)
 {
-   BSON_ASSERT (bulk);
+   BSON_ASSERT_PARAM (bulk);
 
    return bulk->server_id;
 }
@@ -936,7 +976,7 @@ void
 mongoc_bulk_operation_set_hint (mongoc_bulk_operation_t *bulk,
                                 uint32_t server_id)
 {
-   BSON_ASSERT (bulk);
+   BSON_ASSERT_PARAM (bulk);
 
    bulk->server_id = server_id;
 }
@@ -946,7 +986,43 @@ void
 mongoc_bulk_operation_set_bypass_document_validation (
    mongoc_bulk_operation_t *bulk, bool bypass)
 {
-   BSON_ASSERT (bulk);
+   BSON_ASSERT_PARAM (bulk);
 
    bulk->flags.bypass_document_validation = bypass;
 }
+
+
+void
+mongoc_bulk_operation_set_comment (mongoc_bulk_operation_t *bulk,
+                                   const bson_value_t *comment)
+{
+   BSON_ASSERT_PARAM (bulk);
+   BSON_ASSERT_PARAM (comment);
+   BSON_ASSERT (comment->value_type != BSON_TYPE_EOD);
+
+   /* This method cannot be called after appending operations, as the CRUD spec
+    * states the option should apply to all commands. Since commands are
+    * initialized as operations are added, allowing "comment" to be changed at
+    * any time could violate that contract. */
+   BSON_ASSERT (bulk->commands.len == 0);
+
+   bson_value_destroy (&bulk->comment);
+   bson_value_copy (comment, &bulk->comment);
+}
+
+
+void
+mongoc_bulk_operation_set_let (mongoc_bulk_operation_t *bulk, const bson_t *let)
+{
+   BSON_ASSERT_PARAM (bulk);
+   BSON_ASSERT_PARAM (let);
+
+   /* This method cannot be called after appending operations, as the CRUD spec
+    * states the option should apply to all commands (excluding insert). Since
+    * commands are initialized as operations are added, allowing "let" to be
+    * changed at any time could violate that contract. */
+   BSON_ASSERT (bulk->commands.len == 0);
+
+   bson_destroy (&bulk->let);
+   bson_copy_to (let, &bulk->let);
+}
