@@ -42,7 +42,9 @@ struct _mongoc_auto_encryption_opts_t {
    bson_t *kms_providers;
    bson_t *tls_opts;
    bson_t *schema_map;
+   bson_t *encrypted_fields_map;
    bool bypass_auto_encryption;
+   bool bypass_query_analysis;
    bson_t *extra;
 };
 
@@ -61,6 +63,7 @@ mongoc_auto_encryption_opts_destroy (mongoc_auto_encryption_opts_t *opts)
    bson_destroy (opts->extra);
    bson_destroy (opts->kms_providers);
    bson_destroy (opts->schema_map);
+   bson_destroy (opts->encrypted_fields_map);
    bson_free (opts->keyvault_db);
    bson_free (opts->keyvault_coll);
    bson_destroy (opts->tls_opts);
@@ -155,6 +158,20 @@ mongoc_auto_encryption_opts_set_schema_map (mongoc_auto_encryption_opts_t *opts,
    }
 }
 
+void
+mongoc_auto_encryption_opts_set_encrypted_fields_map (
+   mongoc_auto_encryption_opts_t *opts, const bson_t *encrypted_fields_map)
+{
+   if (!opts) {
+      return;
+   }
+   bson_destroy (opts->encrypted_fields_map);
+   opts->encrypted_fields_map = NULL;
+   if (encrypted_fields_map) {
+      opts->encrypted_fields_map = bson_copy (encrypted_fields_map);
+   }
+}
+
 void
 mongoc_auto_encryption_opts_set_bypass_auto_encryption (
    mongoc_auto_encryption_opts_t *opts, bool bypass_auto_encryption)
@@ -165,6 +182,16 @@ mongoc_auto_encryption_opts_set_bypass_auto_encryption (
    opts->bypass_auto_encryption = bypass_auto_encryption;
 }
 
+void
+mongoc_auto_encryption_opts_set_bypass_query_analysis (
+   mongoc_auto_encryption_opts_t *opts, bool bypass_query_analysis)
+{
+   if (!opts) {
+      return;
+   }
+   opts->bypass_query_analysis = bypass_query_analysis;
+}
+
 void
 mongoc_auto_encryption_opts_set_extra (mongoc_auto_encryption_opts_t *opts,
                                        const bson_t *extra)
@@ -268,6 +295,8 @@ struct _mongoc_client_encryption_datakey_opts_t {
    bson_t *masterkey;
    char **keyaltnames;
    uint32_t keyaltnames_count;
+   uint8_t *keymaterial;
+   uint32_t keymaterial_len;
 };
 
 mongoc_client_encryption_datakey_opts_t *
@@ -301,6 +330,7 @@ mongoc_client_encryption_datakey_opts_destroy (
 
    bson_destroy (opts->masterkey);
    _clear_datakey_keyaltnames (opts);
+   bson_free (opts->keymaterial);
 
    bson_free (opts);
 }
@@ -344,6 +374,25 @@ mongoc_client_encryption_datakey_opts_set_keyaltnames (
    }
 }
 
+void
+mongoc_client_encryption_datakey_opts_set_keymaterial (
+   mongoc_client_encryption_datakey_opts_t *opts,
+   const uint8_t *data,
+   uint32_t len)
+{
+   if (!opts) {
+      return;
+   }
+
+   if (opts->keymaterial) {
+      bson_free (opts->keymaterial);
+   }
+
+   opts->keymaterial = bson_malloc (len);
+   memcpy (opts->keymaterial, data, len);
+   opts->keymaterial_len = len;
+}
+
 /*--------------------------------------------------------------------------
  * Explicit Encryption options.
  *--------------------------------------------------------------------------
@@ -352,6 +401,11 @@ struct _mongoc_client_encryption_encrypt_opts_t {
    bson_value_t keyid;
    char *algorithm;
    char *keyaltname;
+   struct {
+      int64_t value;
+      bool set;
+   } contention_factor;
+   char *query_type;
 };
 
 mongoc_client_encryption_encrypt_opts_t *
@@ -370,6 +424,7 @@ mongoc_client_encryption_encrypt_opts_destroy (
    bson_value_destroy (&opts->keyid);
    bson_free (opts->algorithm);
    bson_free (opts->keyaltname);
+   bson_free (opts->query_type);
    bson_free (opts);
 }
 
@@ -411,6 +466,77 @@ mongoc_client_encryption_encrypt_opts_set_algorithm (
    opts->algorithm = bson_strdup (algorithm);
 }
 
+void
+mongoc_client_encryption_encrypt_opts_set_contention_factor (
+   mongoc_client_encryption_encrypt_opts_t *opts, int64_t contention_factor)
+{
+   if (!opts) {
+      return;
+   }
+   opts->contention_factor.value = contention_factor;
+   opts->contention_factor.set = true;
+}
+
+void
+mongoc_client_encryption_encrypt_opts_set_query_type (
+   mongoc_client_encryption_encrypt_opts_t *opts, const char *query_type)
+{
+   if (!opts) {
+      return;
+   }
+   bson_free (opts->query_type);
+   opts->query_type = query_type ? bson_strdup (query_type) : NULL;
+}
+
+/*--------------------------------------------------------------------------
+ * RewrapManyDataKeyResult.
+ *--------------------------------------------------------------------------
+ */
+struct _mongoc_client_encryption_rewrap_many_datakey_result_t {
+   bson_t bulk_write_result;
+};
+
+mongoc_client_encryption_rewrap_many_datakey_result_t *
+mongoc_client_encryption_rewrap_many_datakey_result_new (void)
+{
+   mongoc_client_encryption_rewrap_many_datakey_result_t *const res =
+      bson_malloc0 (
+         sizeof (mongoc_client_encryption_rewrap_many_datakey_result_t));
+
+   bson_init (&res->bulk_write_result);
+
+   return res;
+}
+
+void
+mongoc_client_encryption_rewrap_many_datakey_result_destroy (
+   mongoc_client_encryption_rewrap_many_datakey_result_t *result)
+{
+   if (!result) {
+      return;
+   }
+
+   bson_destroy (&result->bulk_write_result);
+   bson_free (result);
+}
+
+const bson_t *
+mongoc_client_encryption_rewrap_many_datakey_result_get_bulk_write_result (
+   mongoc_client_encryption_rewrap_many_datakey_result_t *result)
+{
+   if (!result) {
+      return NULL;
+   }
+
+   /* bulkWriteResult may be empty if no result of a bulk write operation has
+    * been assigned to it. Treat as equivalent to an unset optional state. */
+   if (bson_empty (&result->bulk_write_result)) {
+      return NULL;
+   }
+
+   return &result->bulk_write_result;
+}
+
 #ifndef MONGOC_ENABLE_CLIENT_SIDE_ENCRYPTION
 
 static bool
@@ -482,6 +608,89 @@ mongoc_client_encryption_create_datakey (
 }
 
 
+bool
+mongoc_client_encryption_rewrap_many_datakey (
+   mongoc_client_encryption_t *client_encryption,
+   const bson_t *filter,
+   const char *provider,
+   const bson_t *master_key,
+   mongoc_client_encryption_rewrap_many_datakey_result_t *result,
+   bson_error_t *error)
+{
+   return _disabled_error (error);
+}
+
+
+bool
+mongoc_client_encryption_delete_key (
+   mongoc_client_encryption_t *client_encryption,
+   const bson_value_t *keyid,
+   bson_t *reply,
+   bson_error_t *error)
+{
+   _mongoc_bson_init_if_set (reply);
+   return _disabled_error (error);
+}
+
+
+bool
+mongoc_client_encryption_get_key (mongoc_client_encryption_t *client_encryption,
+                                  const bson_value_t *keyid,
+                                  bson_t *key_doc,
+                                  bson_error_t *error)
+{
+   _mongoc_bson_init_if_set (key_doc);
+   return _disabled_error (error);
+}
+
+
+mongoc_cursor_t *
+mongoc_client_encryption_get_keys (
+   mongoc_client_encryption_t *client_encryption, bson_error_t *error)
+{
+   _disabled_error (error);
+   return NULL;
+}
+
+
+bool
+mongoc_client_encryption_add_key_alt_name (
+   mongoc_client_encryption_t *client_encryption,
+   const bson_value_t *keyid,
+   const char *keyaltname,
+   bson_t *key_doc,
+   bson_error_t *error)
+{
+   _mongoc_bson_init_if_set (key_doc);
+   return _disabled_error (error);
+}
+
+
+bool
+mongoc_client_encryption_remove_key_alt_name (
+   mongoc_client_encryption_t *client_encryption,
+   const bson_value_t *keyid,
+   const char *keyaltname,
+   bson_t *key_doc,
+   bson_error_t *error)
+{
+   _mongoc_bson_init_if_set (key_doc);
+   return _disabled_error (error);
+}
+
+
+bool
+mongoc_client_encryption_get_key_by_alt_name (
+   mongoc_client_encryption_t *client_encryption,
+   const char *keyaltname,
+   bson_t *key_doc,
+   bson_error_t *error)
+{
+   _mongoc_bson_init_if_set (key_doc);
+   return _disabled_error (error);
+}
+
+
 MONGOC_EXPORT (mongoc_client_encryption_t *)
 mongoc_client_encryption_new (mongoc_client_encryption_opts_t *opts,
                               bson_error_t *error)
@@ -597,9 +806,13 @@ _release_mongocryptd_client (mongoc_client_t *client_encrypted,
 mongoc_collection_t *
 _get_keyvault_coll (mongoc_client_t *client_encrypted)
 {
+   mongoc_write_concern_t *const wc = mongoc_write_concern_new ();
+   mongoc_read_concern_t *const rc = mongoc_read_concern_new ();
+
    mongoc_client_t *keyvault_client;
    const char *db;
    const char *coll;
+   mongoc_collection_t *res = NULL;
 
    db = client_encrypted->topology->keyvault_db;
    coll = client_encrypted->topology->keyvault_coll;
@@ -618,7 +831,19 @@ _get_keyvault_coll (mongoc_client_t *client_encrypted)
          keyvault_client = client_encrypted;
       }
    }
-   return mongoc_client_get_collection (keyvault_client, db, coll);
+
+   res = mongoc_client_get_collection (keyvault_client, db, coll);
+
+   mongoc_write_concern_set_w (wc, MONGOC_WRITE_CONCERN_W_MAJORITY);
+   mongoc_collection_set_write_concern (res, wc);
+
+   mongoc_read_concern_set_level (rc, MONGOC_READ_CONCERN_LEVEL_MAJORITY);
+   mongoc_collection_set_read_concern (res, rc);
+
+   mongoc_write_concern_destroy (wc);
+   mongoc_read_concern_destroy (rc);
+
+   return res;
 }
 
 void
@@ -888,6 +1113,7 @@ _do_spawn (const char *path, char **args, bson_error_t *error)
 }
 #else
 
+
 /*--------------------------------------------------------------------------
  *
  * _do_spawn --
@@ -928,6 +1154,13 @@ _do_spawn (const char *path, char **args, bson_error_t *error)
    int fd;
    char *to_exec;
 
+   // String allocation must be done up-front, as allocation is not fork-safe.
+   if (path) {
+      to_exec = bson_strdup_printf ("%s%s", path, args[0]);
+   } else {
+      to_exec = bson_strdup (args[0]);
+   }
+
    /* Fork. The child will terminate immediately (after fork-exec'ing
     * mongocryptd). This orphans mongocryptd, and allows parent to wait on
     * child. */
@@ -939,6 +1172,7 @@ _do_spawn (const char *path, char **args, bson_error_t *error)
                       "failed to fork (errno=%d) '%s'",
                       errno,
                       strerror (errno));
+      bson_free (to_exec);
       return false;
    } else if (pid > 0) {
       int child_status;
@@ -952,9 +1186,11 @@ _do_spawn (const char *path, char **args, bson_error_t *error)
                          "failed to wait for child (errno=%d) '%s'",
                          errno,
                          strerror (errno));
+         bson_free (to_exec);
          return false;
       }
       /* parent is done at this point, return. */
+      bson_free (to_exec);
       return true;
    }
 
@@ -967,7 +1203,7 @@ _do_spawn (const char *path, char **args, bson_error_t *error)
    /* Start a new session for the child, so it is not bound to the current
     * session (e.g. terminal session). */
    if (setsid () < 0) {
-      exit (EXIT_FAILURE);
+      _exit (EXIT_FAILURE);
    }
 
    /* Fork again. Child terminates so mongocryptd gets orphaned and immedately
@@ -975,10 +1211,10 @@ _do_spawn (const char *path, char **args, bson_error_t *error)
    signal (SIGHUP, SIG_IGN);
    pid = fork ();
    if (pid < 0) {
-      exit (EXIT_FAILURE);
+      _exit (EXIT_FAILURE);
    } else if (pid > 0) {
       /* Child terminates immediately. */
-      exit (EXIT_SUCCESS);
+      _exit (EXIT_SUCCESS);
    }
 
    /* If we later decide to change the working directory for the pid file path,
@@ -992,7 +1228,7 @@ _do_spawn (const char *path, char **args, bson_error_t *error)
    /* Close and reopen stdin. */
    fd = open ("/dev/null", O_RDONLY);
    if (fd < 0) {
-      exit (EXIT_FAILURE);
+      _exit (EXIT_FAILURE);
    }
    dup2 (fd, STDIN_FILENO);
    close (fd);
@@ -1000,29 +1236,24 @@ _do_spawn (const char *path, char **args, bson_error_t *error)
    /* Close and reopen stdout. */
    fd = open ("/dev/null", O_WRONLY);
    if (fd < 0) {
-      exit (EXIT_FAILURE);
+      _exit (EXIT_FAILURE);
    }
    if (dup2 (fd, STDOUT_FILENO) < 0 || close (fd) < 0) {
-      exit (EXIT_FAILURE);
+      _exit (EXIT_FAILURE);
    }
 
    /* Close and reopen stderr. */
    fd = open ("/dev/null", O_RDWR);
    if (fd < 0) {
-      exit (EXIT_FAILURE);
+      _exit (EXIT_FAILURE);
    }
    if (dup2 (fd, STDERR_FILENO) < 0 || close (fd) < 0) {
-      exit (EXIT_FAILURE);
+      _exit (EXIT_FAILURE);
    }
 
-   if (path) {
-      to_exec = bson_strdup_printf ("%s%s", path, args[0]);
-   } else {
-      to_exec = bson_strdup (args[0]);
-   }
    if (execvp (to_exec, args) < 0) {
       /* Need to exit. */
-      exit (EXIT_FAILURE);
+      _exit (EXIT_FAILURE);
    }
 
    /* Will never execute. */
@@ -1181,6 +1412,34 @@ _parse_extra (const bson_t *extra,
             GOTO (fail);
          }
       }
+
+      if (bson_iter_init_find (&iter, extra, "cryptSharedLibPath")) {
+         if (!BSON_ITER_HOLDS_UTF8 (&iter)) {
+            bson_set_error (error,
+                            MONGOC_ERROR_CLIENT,
+                            MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_ARG,
+                            "Expected a string for 'cryptSharedLibPath'");
+            GOTO (fail);
+         }
+         size_t len;
+         const char *ptr = bson_iter_utf8_unsafe (&iter, &len);
+         bson_free (topology->clientSideEncryption.autoOptions.extraOptions
+                       .cryptSharedLibPath);
+         topology->clientSideEncryption.autoOptions.extraOptions
+            .cryptSharedLibPath = bson_strdup (ptr);
+      }
+
+      if (bson_iter_init_find (&iter, extra, "cryptSharedLibRequired")) {
+         if (!BSON_ITER_HOLDS_BOOL (&iter)) {
+            bson_set_error (error,
+                            MONGOC_ERROR_CLIENT,
+                            MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_ARG,
+                            "Expected a bool for 'cryptSharedLibRequired'");
+            GOTO (fail);
+         }
+         topology->clientSideEncryption.autoOptions.extraOptions
+            .cryptSharedLibRequired = bson_iter_bool_unsafe (&iter);
+      }
    }
 
 
@@ -1281,15 +1540,27 @@ _mongoc_cse_client_enable_auto_encryption (mongoc_client_t *client,
       GOTO (fail);
    }
 
-   client->topology->crypt = _mongoc_crypt_new (
-      opts->kms_providers, opts->schema_map, opts->tls_opts, error);
+   client->topology->crypt =
+      _mongoc_crypt_new (opts->kms_providers,
+                         opts->schema_map,
+                         opts->encrypted_fields_map,
+                         opts->tls_opts,
+                         client->topology->clientSideEncryption.autoOptions
+                            .extraOptions.cryptSharedLibPath,
+                         client->topology->clientSideEncryption.autoOptions
+                            .extraOptions.cryptSharedLibRequired,
+                         opts->bypass_auto_encryption,
+                         opts->bypass_query_analysis,
+                         error);
    if (!client->topology->crypt) {
       GOTO (fail);
    }
 
    client->topology->bypass_auto_encryption = opts->bypass_auto_encryption;
+   client->topology->bypass_query_analysis = opts->bypass_query_analysis;
 
-   if (!client->topology->bypass_auto_encryption) {
+   if (!client->topology->bypass_auto_encryption &&
+       !client->topology->bypass_query_analysis) {
       if (!client->topology->mongocryptd_bypass_spawn) {
          if (!_spawn_mongocryptd (client->topology->mongocryptd_spawn_path,
                                   client->topology->mongocryptd_spawn_args,
@@ -1344,6 +1615,11 @@ _mongoc_cse_client_enable_auto_encryption (mongoc_client_t *client,
       client->topology->keyvault_client = opts->keyvault_client;
    }
 
+   if (opts->encrypted_fields_map) {
+      client->topology->encrypted_fields_map =
+         bson_copy (opts->encrypted_fields_map);
+   }
+
    ret = true;
 fail:
    mongoc_uri_destroy (mongocryptd_uri);
@@ -1425,15 +1701,26 @@ _mongoc_cse_client_pool_enable_auto_encryption (
       GOTO (fail);
    }
 
-   topology->crypt = _mongoc_crypt_new (
-      opts->kms_providers, opts->schema_map, opts->tls_opts, error);
+   topology->crypt =
+      _mongoc_crypt_new (opts->kms_providers,
+                         opts->schema_map,
+                         opts->encrypted_fields_map,
+                         opts->tls_opts,
+                         topology->clientSideEncryption.autoOptions.extraOptions
+                            .cryptSharedLibPath,
+                         topology->clientSideEncryption.autoOptions.extraOptions
+                            .cryptSharedLibRequired,
+                         opts->bypass_auto_encryption,
+                         opts->bypass_query_analysis,
+                         error);
    if (!topology->crypt) {
       GOTO (fail);
    }
 
    topology->bypass_auto_encryption = opts->bypass_auto_encryption;
+   topology->bypass_query_analysis = opts->bypass_query_analysis;
 
-   if (!topology->bypass_auto_encryption) {
+   if (!topology->bypass_auto_encryption && !topology->bypass_query_analysis) {
       if (!topology->mongocryptd_bypass_spawn) {
          if (!_spawn_mongocryptd (topology->mongocryptd_spawn_path,
                                   topology->mongocryptd_spawn_args,
@@ -1460,6 +1747,10 @@ _mongoc_cse_client_pool_enable_auto_encryption (
       topology->keyvault_client_pool = opts->keyvault_client_pool;
    }
 
+   if (opts->encrypted_fields_map) {
+      topology->encrypted_fields_map = bson_copy (opts->encrypted_fields_map);
+   }
+
    setup_okay = true;
    BSON_ASSERT (prev_cse_state == MONGOC_CSE_DISABLED);
 fail:
@@ -1487,6 +1778,7 @@ mongoc_client_encryption_new (mongoc_client_encryption_opts_t *opts,
    mongoc_client_encryption_t *client_encryption = NULL;
    bool success = false;
    mongoc_write_concern_t *wc = NULL;
+   mongoc_read_concern_t *rc = NULL;
 
    /* Check for required options */
    if (!opts || !opts->keyvault_client || !opts->keyvault_db ||
@@ -1510,12 +1802,23 @@ mongoc_client_encryption_new (mongoc_client_encryption_opts_t *opts,
    client_encryption->keyvault_coll = mongoc_client_get_collection (
       opts->keyvault_client, opts->keyvault_db, opts->keyvault_coll);
    wc = mongoc_write_concern_new ();
-   mongoc_write_concern_set_wmajority (wc, 1000);
+   mongoc_write_concern_set_w (wc, MONGOC_WRITE_CONCERN_W_MAJORITY);
    mongoc_collection_set_write_concern (client_encryption->keyvault_coll, wc);
+   rc = mongoc_read_concern_new ();
+   mongoc_read_concern_set_level (rc, MONGOC_READ_CONCERN_LEVEL_MAJORITY);
+   mongoc_collection_set_read_concern (client_encryption->keyvault_coll, rc);
 
    client_encryption->kms_providers = bson_copy (opts->kms_providers);
-   client_encryption->crypt = _mongoc_crypt_new (
-      opts->kms_providers, NULL /* schema_map */, opts->tls_opts, error);
+   client_encryption->crypt =
+      _mongoc_crypt_new (opts->kms_providers,
+                         NULL /* schema_map */,
+                         NULL /* encrypted_fields_map */,
+                         opts->tls_opts,
+                         NULL /* No crypt_shared path */,
+                         false /* crypt_shared not requried */,
+                         true, /* bypassAutoEncryption (We are explicit) */
+                         false /* bypass_query_analysis. Not applicable. */,
+                         error);
    if (!client_encryption->crypt) {
       goto fail;
    }
@@ -1523,6 +1826,7 @@ mongoc_client_encryption_new (mongoc_client_encryption_opts_t *opts,
 
 fail:
    mongoc_write_concern_destroy (wc);
+   mongoc_read_concern_destroy (rc);
    if (!success) {
       mongoc_client_encryption_destroy (client_encryption);
       return NULL;
@@ -1542,6 +1846,23 @@ mongoc_client_encryption_destroy (mongoc_client_encryption_t *client_encryption)
    bson_free (client_encryption);
 }
 
+static bool
+_coll_has_write_concern_majority (const mongoc_collection_t *coll)
+{
+   const mongoc_write_concern_t *const wc =
+      mongoc_collection_get_write_concern (coll);
+   return wc && mongoc_write_concern_get_wmajority (wc);
+}
+
+static bool
+_coll_has_read_concern_majority (const mongoc_collection_t *coll)
+{
+   const mongoc_read_concern_t *const rc =
+      mongoc_collection_get_read_concern (coll);
+   const char *const level = rc ? mongoc_read_concern_get_level (rc) : NULL;
+   return level && strcmp (level, MONGOC_READ_CONCERN_LEVEL_MAJORITY) == 0;
+}
+
 bool
 mongoc_client_encryption_create_datakey (
    mongoc_client_encryption_t *client_encryption,
@@ -1556,6 +1877,11 @@ mongoc_client_encryption_create_datakey (
 
    ENTRY;
 
+   BSON_ASSERT_PARAM (client_encryption);
+
+   BSON_ASSERT (
+      _coll_has_write_concern_majority (client_encryption->keyvault_coll));
+
    if (!opts) {
       bson_set_error (error,
                       MONGOC_ERROR_CLIENT,
@@ -1576,12 +1902,13 @@ mongoc_client_encryption_create_datakey (
                                       opts->masterkey,
                                       opts->keyaltnames,
                                       opts->keyaltnames_count,
+                                      opts->keymaterial,
+                                      opts->keymaterial_len,
                                       &datakey,
                                       error)) {
       GOTO (fail);
    }
 
-   /* Insert the data key with write concern majority */
    if (!mongoc_collection_insert_one (client_encryption->keyvault_coll,
                                       &datakey,
                                       NULL /* opts */,
@@ -1621,6 +1948,508 @@ fail:
    RETURN (ret);
 }
 
+bool
+mongoc_client_encryption_rewrap_many_datakey (
+   mongoc_client_encryption_t *client_encryption,
+   const bson_t *filter,
+   const char *provider,
+   const bson_t *master_key,
+   mongoc_client_encryption_rewrap_many_datakey_result_t *result,
+   bson_error_t *error)
+{
+   bson_t keys = BSON_INITIALIZER;
+   bson_t local_result = BSON_INITIALIZER;
+   bson_t *const bulk_write_result =
+      result ? &result->bulk_write_result : &local_result;
+   mongoc_bulk_operation_t *bulk = NULL;
+   bson_iter_t iter;
+   bool ret = false;
+
+   ENTRY;
+
+   BSON_ASSERT_PARAM (client_encryption);
+
+   BSON_ASSERT (
+      _coll_has_read_concern_majority (client_encryption->keyvault_coll));
+   BSON_ASSERT (
+      _coll_has_write_concern_majority (client_encryption->keyvault_coll));
+
+   bson_reinit (bulk_write_result);
+
+   if (!_mongoc_crypt_rewrap_many_datakey (client_encryption->crypt,
+                                           client_encryption->keyvault_coll,
+                                           filter,
+                                           provider,
+                                           master_key,
+                                           &keys,
+                                           error)) {
+      GOTO (fail);
+   }
+
+   /* No keys rewrapped, no key documents to update. */
+   if (bson_empty (&keys)) {
+      bson_destroy (&keys);
+      bson_destroy (&local_result);
+      return true;
+   }
+
+   bulk = mongoc_collection_create_bulk_operation_with_opts (
+      client_encryption->keyvault_coll, NULL);
+
+   BSON_ASSERT (bulk);
+
+   if (!bson_iter_init_find (&iter, &keys, "v")) {
+      bson_set_error (error,
+                      MONGOC_ERROR_CLIENT,
+                      MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_STATE,
+                      "result did not contain expected field 'v'");
+      GOTO (fail);
+   }
+
+   if (!BSON_ITER_HOLDS_ARRAY (&iter)) {
+      bson_set_error (error,
+                      MONGOC_ERROR_CLIENT,
+                      MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_STATE,
+                      "result did not return an array as expected");
+      GOTO (fail);
+   }
+
+   BSON_ASSERT (bson_iter_recurse (&iter, &iter));
+
+   while (bson_iter_next (&iter)) {
+      const uint8_t *data = NULL;
+      uint32_t len = 0u;
+      bson_t key;
+      bson_iter_t key_iter;
+      bson_subtype_t subtype;
+      bson_t selector = BSON_INITIALIZER;
+      bson_t document = BSON_INITIALIZER;
+      bool doc_success = false;
+
+      bson_iter_document (&iter, &len, &data);
+
+      if (!data || !bson_init_static (&key, data, len)) {
+         bson_set_error (error,
+                         MONGOC_ERROR_CLIENT,
+                         MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_STATE,
+                         "element is not a valid BSON document");
+         goto doc_done;
+      }
+
+      /* Find _id and use as selector. */
+      {
+         if (!bson_iter_init_find (&key_iter, &key, "_id")) {
+            bson_set_error (error,
+                            MONGOC_ERROR_CLIENT,
+                            MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_STATE,
+                            "could not find _id in key document");
+            goto doc_done;
+         }
+
+         bson_iter_binary (&key_iter, &subtype, &len, &data);
+
+         if (!data || subtype != BSON_SUBTYPE_UUID) {
+            bson_set_error (error,
+                            MONGOC_ERROR_CLIENT,
+                            MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_STATE,
+                            "expected _id in key document to be a UUID");
+            goto doc_done;
+         }
+
+         BSON_ASSERT (bson_append_iter (&selector, "_id", 3, &key_iter));
+      }
+
+      /* Find and include potentially updated fields. */
+      {
+         bson_t child;
+
+         BSON_ASSERT (BSON_APPEND_DOCUMENT_BEGIN (&document, "$set", &child));
+         {
+            if (bson_iter_init_find (&key_iter, &key, "masterKey")) {
+               BSON_ASSERT (
+                  bson_append_iter (&child, "masterKey", -1, &key_iter));
+            }
+
+            if (bson_iter_init_find (&key_iter, &key, "keyMaterial")) {
+               BSON_ASSERT (
+                  bson_append_iter (&child, "keyMaterial", -1, &key_iter));
+            }
+         }
+         BSON_ASSERT (bson_append_document_end (&document, &child));
+      }
+
+      /* Update updateDate field. */
+      BCON_APPEND (
+         &document, "$currentDate", "{", "updateDate", BCON_BOOL (true), "}");
+
+      if (!mongoc_bulk_operation_update_one_with_opts (
+             bulk, &selector, &document, NULL, error)) {
+         goto doc_done;
+      }
+
+      doc_success = true;
+
+   doc_done:
+      bson_destroy (&key);
+      bson_destroy (&selector);
+      bson_destroy (&document);
+
+      if (!doc_success) {
+         GOTO (fail);
+      }
+   }
+
+   if (!mongoc_bulk_operation_execute (bulk, bulk_write_result, error)) {
+      GOTO (fail);
+   }
+
+   ret = true;
+
+fail:
+   bson_destroy (&keys);
+   bson_destroy (&local_result);
+   mongoc_bulk_operation_destroy (bulk);
+
+   RETURN (ret);
+}
+
+bool
+mongoc_client_encryption_delete_key (
+   mongoc_client_encryption_t *client_encryption,
+   const bson_value_t *keyid,
+   bson_t *reply,
+   bson_error_t *error)
+{
+   bool ret = false;
+   bson_t selector = BSON_INITIALIZER;
+
+   ENTRY;
+
+   BSON_ASSERT_PARAM (client_encryption);
+   BSON_ASSERT_PARAM (keyid);
+
+   BSON_ASSERT (
+      _coll_has_write_concern_majority (client_encryption->keyvault_coll));
+
+   BSON_ASSERT (keyid->value_type == BSON_TYPE_BINARY);
+   BSON_ASSERT (keyid->value.v_binary.subtype == BSON_SUBTYPE_UUID);
+   BSON_ASSERT (keyid->value.v_binary.data_len > 0u);
+
+   BSON_ASSERT (BSON_APPEND_BINARY (&selector,
+                                    "_id",
+                                    keyid->value.v_binary.subtype,
+                                    keyid->value.v_binary.data,
+                                    keyid->value.v_binary.data_len));
+
+   ret = mongoc_collection_delete_one (
+      client_encryption->keyvault_coll, &selector, NULL, reply, error);
+
+   bson_destroy (&selector);
+
+   RETURN (ret);
+}
+
+bool
+mongoc_client_encryption_get_key (mongoc_client_encryption_t *client_encryption,
+                                  const bson_value_t *keyid,
+                                  bson_t *key_doc,
+                                  bson_error_t *error)
+{
+   bson_t filter = BSON_INITIALIZER;
+   mongoc_cursor_t *cursor = NULL;
+   bool ret = false;
+
+   ENTRY;
+
+   BSON_ASSERT_PARAM (client_encryption);
+   BSON_ASSERT_PARAM (keyid);
+
+   BSON_ASSERT (keyid->value_type == BSON_TYPE_BINARY);
+   BSON_ASSERT (keyid->value.v_binary.subtype == BSON_SUBTYPE_UUID);
+   BSON_ASSERT (keyid->value.v_binary.data_len > 0u);
+
+   BSON_ASSERT (BSON_APPEND_BINARY (&filter,
+                                    "_id",
+                                    keyid->value.v_binary.subtype,
+                                    keyid->value.v_binary.data,
+                                    keyid->value.v_binary.data_len));
+
+   BSON_ASSERT (
+      _coll_has_read_concern_majority (client_encryption->keyvault_coll));
+
+   _mongoc_bson_init_if_set (key_doc);
+
+   cursor = mongoc_collection_find_with_opts (
+      client_encryption->keyvault_coll, &filter, NULL, NULL);
+
+   ret = !mongoc_cursor_error (cursor, error);
+
+   if (ret && key_doc) {
+      const bson_t *bson = NULL;
+
+      if (mongoc_cursor_next (cursor, &bson)) {
+         bson_copy_to (bson, key_doc);
+      } else if (mongoc_cursor_error (cursor, error)) {
+         ret = false;
+      }
+   }
+
+   bson_destroy (&filter);
+   mongoc_cursor_destroy (cursor);
+
+   RETURN (ret);
+}
+
+mongoc_cursor_t *
+mongoc_client_encryption_get_keys (
+   mongoc_client_encryption_t *client_encryption, bson_error_t *error)
+{
+   mongoc_cursor_t *cursor = NULL;
+   bson_t filter = BSON_INITIALIZER;
+
+   ENTRY;
+
+   BSON_ASSERT_PARAM (client_encryption);
+
+   BSON_ASSERT (
+      _coll_has_read_concern_majority (client_encryption->keyvault_coll));
+
+   /* If an error occurred, user should query cursor error. */
+   cursor = mongoc_collection_find_with_opts (
+      client_encryption->keyvault_coll, &filter, NULL, NULL);
+
+   bson_destroy (&filter);
+
+   RETURN (cursor);
+}
+
+bool
+mongoc_client_encryption_add_key_alt_name (
+   mongoc_client_encryption_t *client_encryption,
+   const bson_value_t *keyid,
+   const char *keyaltname,
+   bson_t *key_doc,
+   bson_error_t *error)
+{
+   mongoc_find_and_modify_opts_t *const opts =
+      mongoc_find_and_modify_opts_new ();
+   bson_t query = BSON_INITIALIZER;
+   bool ret = false;
+   bson_t local_reply;
+
+   ENTRY;
+
+   BSON_ASSERT_PARAM (client_encryption);
+   BSON_ASSERT_PARAM (keyid);
+   BSON_ASSERT_PARAM (keyaltname);
+
+   BSON_ASSERT (
+      _coll_has_read_concern_majority (client_encryption->keyvault_coll));
+   BSON_ASSERT (
+      _coll_has_write_concern_majority (client_encryption->keyvault_coll));
+
+   BSON_ASSERT (keyid->value_type == BSON_TYPE_BINARY);
+   BSON_ASSERT (keyid->value.v_binary.subtype == BSON_SUBTYPE_UUID);
+   BSON_ASSERT (keyid->value.v_binary.data_len > 0u);
+
+   BSON_ASSERT (BSON_APPEND_BINARY (&query,
+                                    "_id",
+                                    keyid->value.v_binary.subtype,
+                                    keyid->value.v_binary.data,
+                                    keyid->value.v_binary.data_len));
+
+   _mongoc_bson_init_if_set (key_doc);
+
+   {
+      bson_t *const update = BCON_NEW (
+         "$addToSet", "{", "keyAltNames", BCON_UTF8 (keyaltname), "}");
+      BSON_ASSERT (mongoc_find_and_modify_opts_set_update (opts, update));
+      bson_destroy (update);
+   }
+
+   ret = mongoc_collection_find_and_modify_with_opts (
+      client_encryption->keyvault_coll, &query, opts, &local_reply, error);
+
+   if (ret && key_doc) {
+      bson_iter_t iter;
+
+      if (bson_iter_init_find (&iter, &local_reply, "value")) {
+         const bson_value_t *const value = bson_iter_value (&iter);
+
+         if (value->value_type == BSON_TYPE_DOCUMENT) {
+            bson_t bson;
+            BSON_ASSERT (bson_init_static (
+               &bson, value->value.v_doc.data, value->value.v_doc.data_len));
+            bson_copy_to (&bson, key_doc);
+            bson_destroy (&bson);
+         } else if (value->value_type == BSON_TYPE_NULL) {
+            bson_t bson = BSON_INITIALIZER;
+            bson_copy_to (&bson, key_doc);
+            bson_destroy (&bson);
+         } else {
+            bson_set_error (error,
+                            MONGOC_ERROR_CLIENT,
+                            MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_STATE,
+                            "expected field value to be a document or null");
+            ret = false;
+         }
+      }
+   }
+
+   mongoc_find_and_modify_opts_destroy (opts);
+   bson_destroy (&query);
+   bson_destroy (&local_reply);
+
+   RETURN (ret);
+}
+
+bool
+mongoc_client_encryption_remove_key_alt_name (
+   mongoc_client_encryption_t *client_encryption,
+   const bson_value_t *keyid,
+   const char *keyaltname,
+   bson_t *key_doc,
+   bson_error_t *error)
+{
+   bson_t query = BSON_INITIALIZER;
+   bool ret = false;
+   bson_t local_reply;
+
+   ENTRY;
+
+   BSON_ASSERT_PARAM (client_encryption);
+   BSON_ASSERT_PARAM (keyid);
+   BSON_ASSERT_PARAM (keyaltname);
+
+   BSON_ASSERT (
+      _coll_has_write_concern_majority (client_encryption->keyvault_coll));
+
+   BSON_ASSERT (keyid->value_type == BSON_TYPE_BINARY);
+   BSON_ASSERT (keyid->value.v_binary.subtype == BSON_SUBTYPE_UUID);
+   BSON_ASSERT (keyid->value.v_binary.data_len > 0u);
+
+   BSON_ASSERT (BSON_APPEND_BINARY (&query,
+                                    "_id",
+                                    keyid->value.v_binary.subtype,
+                                    keyid->value.v_binary.data,
+                                    keyid->value.v_binary.data_len));
+
+   _mongoc_bson_init_if_set (key_doc);
+
+
+   {
+      mongoc_find_and_modify_opts_t *const opts =
+         mongoc_find_and_modify_opts_new ();
+
+      /* clang-format off */
+      bson_t *const update = BCON_NEW (
+         "0", "{",
+            "$set", "{",
+               "keyAltNames", "{",
+                  "$cond", "[",
+                     "{",
+                        "$eq", "[", "$keyAltNames", "[", keyaltname, "]", "]",
+                     "}",
+                     "$$REMOVE",
+                     "{",
+                        "$filter", "{",
+                           "input", "$keyAltNames",
+                           "cond", "{",
+                              "$ne", "[", "$$this", keyaltname, "]",
+                           "}",
+                        "}",
+                     "}",
+                  "]",
+               "}",
+            "}",
+         "}");
+      /* clang-format on */
+
+      BSON_ASSERT (mongoc_find_and_modify_opts_set_update (opts, update));
+
+      ret = mongoc_collection_find_and_modify_with_opts (
+         client_encryption->keyvault_coll, &query, opts, &local_reply, error);
+
+      bson_destroy (update);
+      mongoc_find_and_modify_opts_destroy (opts);
+   }
+
+   if (ret && key_doc) {
+      bson_iter_t iter;
+
+      if (bson_iter_init_find (&iter, &local_reply, "value")) {
+         const bson_value_t *const value = bson_iter_value (&iter);
+
+         if (value->value_type == BSON_TYPE_DOCUMENT) {
+            bson_t bson;
+            BSON_ASSERT (bson_init_static (
+               &bson, value->value.v_doc.data, value->value.v_doc.data_len));
+            bson_copy_to (&bson, key_doc);
+            bson_destroy (&bson);
+         } else if (value->value_type == BSON_TYPE_NULL) {
+            bson_t bson = BSON_INITIALIZER;
+            bson_copy_to (&bson, key_doc);
+            bson_destroy (&bson);
+         } else {
+            bson_set_error (error,
+                            MONGOC_ERROR_CLIENT,
+                            MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_STATE,
+                            "expected field value to be a document or null");
+            ret = false;
+         }
+      }
+   }
+
+   bson_destroy (&query);
+   bson_destroy (&local_reply);
+
+   RETURN (ret);
+}
+
+bool
+mongoc_client_encryption_get_key_by_alt_name (
+   mongoc_client_encryption_t *client_encryption,
+   const char *keyaltname,
+   bson_t *key_doc,
+   bson_error_t *error)
+{
+   bson_t filter = BSON_INITIALIZER;
+   mongoc_cursor_t *cursor = NULL;
+   bool ret = false;
+
+   ENTRY;
+
+   BSON_ASSERT_PARAM (client_encryption);
+   BSON_ASSERT_PARAM (keyaltname);
+
+   BSON_ASSERT (
+      _coll_has_write_concern_majority (client_encryption->keyvault_coll));
+
+   BSON_ASSERT (BSON_APPEND_UTF8 (&filter, "keyAltNames", keyaltname));
+
+   _mongoc_bson_init_if_set (key_doc);
+
+   cursor = mongoc_collection_find_with_opts (
+      client_encryption->keyvault_coll, &filter, NULL, NULL);
+
+   ret = !mongoc_cursor_error (cursor, error);
+
+   if (ret && key_doc) {
+      const bson_t *bson = NULL;
+
+      if (mongoc_cursor_next (cursor, &bson)) {
+         bson_copy_to (bson, key_doc);
+      } else if (mongoc_cursor_error (cursor, error)) {
+         ret = false;
+      }
+   }
+
+   bson_destroy (&filter);
+   mongoc_cursor_destroy (cursor);
+
+   RETURN (ret);
+}
+
 bool
 mongoc_client_encryption_encrypt (mongoc_client_encryption_t *client_encryption,
                                   const bson_value_t *value,
@@ -1653,14 +2482,17 @@ mongoc_client_encryption_encrypt (mongoc_client_encryption_t *client_encryption,
       GOTO (fail);
    }
 
-   if (!_mongoc_crypt_explicit_encrypt (client_encryption->crypt,
-                                        client_encryption->keyvault_coll,
-                                        opts->algorithm,
-                                        &opts->keyid,
-                                        opts->keyaltname,
-                                        value,
-                                        ciphertext,
-                                        error)) {
+   if (!_mongoc_crypt_explicit_encrypt (
+          client_encryption->crypt,
+          client_encryption->keyvault_coll,
+          opts->algorithm,
+          &opts->keyid,
+          opts->keyaltname,
+          opts->query_type,
+          opts->contention_factor.set ? &opts->contention_factor.value : NULL,
+          value,
+          ciphertext,
+          error)) {
       GOTO (fail);
    }
 
