@@ -13,22 +13,22 @@
  * limitations under the License.
  */
 
-#include "mongoc/mongoc-config.h"
+#include "mongoc-config.h"
 
 #ifdef MONGOC_ENABLE_CRYPTO
 
 #include <string.h>
 
-#include "mongoc/mongoc-error.h"
-#include "mongoc/mongoc-scram-private.h"
-#include "mongoc/mongoc-rand-private.h"
-#include "mongoc/mongoc-util-private.h"
-#include "mongoc/mongoc-trace-private.h"
+#include "mongoc-error.h"
+#include "mongoc-scram-private.h"
+#include "mongoc-rand-private.h"
+#include "mongoc-util-private.h"
+#include "mongoc-trace-private.h"
 
-#include "mongoc/mongoc-crypto-private.h"
+#include "mongoc-crypto-private.h"
 #include "common-b64-private.h"
 
-#include "mongoc/mongoc-memcmp-private.h"
+#include "mongoc-memcmp-private.h"
 
 #define MONGOC_SCRAM_SERVER_KEY "Server Key"
 #define MONGOC_SCRAM_CLIENT_KEY "Client Key"
@@ -193,6 +193,8 @@ _mongoc_scram_destroy (mongoc_scram_t *scram)
    if (scram->cache) {
       _mongoc_scram_cache_destroy (scram->cache);
    }
+
+   memset (scram, 0, sizeof *scram);
 }
 
 
@@ -289,10 +291,10 @@ _mongoc_scram_start (mongoc_scram_t *scram,
       goto FAIL;
    }
 
-   scram->encoded_nonce_len = bson_b64_ntop (nonce,
-                                             sizeof (nonce),
-                                             scram->encoded_nonce,
-                                             sizeof (scram->encoded_nonce));
+   scram->encoded_nonce_len = mcommon_b64_ntop (nonce,
+                                                sizeof (nonce),
+                                                scram->encoded_nonce,
+                                                sizeof (scram->encoded_nonce));
 
    if (-1 == scram->encoded_nonce_len) {
       bson_set_error (error,
@@ -486,10 +488,10 @@ _mongoc_scram_generate_client_proof (mongoc_scram_t *scram,
       client_proof[i] = scram->client_key[i] ^ client_signature[i];
    }
 
-   r = bson_b64_ntop (client_proof,
-                      _scram_hash_size (scram),
-                      (char *) outbuf + *outbuflen,
-                      outbufmax - *outbuflen);
+   r = mcommon_b64_ntop (client_proof,
+                         _scram_hash_size (scram),
+                         (char *) outbuf + *outbuflen,
+                         outbufmax - *outbuflen);
 
    if (-1 == r) {
       return false;
@@ -701,7 +703,7 @@ _mongoc_scram_step2 (mongoc_scram_t *scram,
    }
 
    decoded_salt_len =
-      bson_b64_pton ((char *) val_s, decoded_salt, sizeof (decoded_salt));
+      mcommon_b64_pton ((char *) val_s, decoded_salt, sizeof (decoded_salt));
 
    if (-1 == decoded_salt_len) {
       bson_set_error (error,
@@ -836,10 +838,10 @@ _mongoc_scram_verify_server_signature (mongoc_scram_t *scram,
                        server_signature);
 
    encoded_server_signature_len =
-      bson_b64_ntop (server_signature,
-                     _scram_hash_size (scram),
-                     encoded_server_signature,
-                     sizeof (encoded_server_signature));
+      mcommon_b64_ntop (server_signature,
+                        _scram_hash_size (scram),
+                        encoded_server_signature,
+                        sizeof (encoded_server_signature));
    if (encoded_server_signature_len == -1) {
       return false;
    }
