@@ -55,6 +55,8 @@ _mongoc_monitor_legacy_write (mongoc_client_t *client,
       command->operation_id,
       &stream->sd->host,
       stream->sd->id,
+      &stream->sd->service_id,
+      stream->sd->server_connection_id,
       NULL,
       client->apm_context);
 
@@ -105,6 +107,8 @@ _mongoc_monitor_legacy_write_succeeded (mongoc_client_t *client,
       command->operation_id,
       &stream->sd->host,
       stream->sd->id,
+      &stream->sd->service_id,
+      stream->sd->server_connection_id,
       false,
       client->apm_context);
 
