@@ -712,7 +712,7 @@ _test_heartbeat_events (bool pooled, bool succeeded)
       client, "admin", tmp_bson ("{'foo': 1}"), NULL, NULL, &error);
 
    /* topology scanner calls hello once */
-   request = mock_server_receives_legacy_hello (server, NULL);
+   request = mock_server_receives_any_hello (server);
 
    if (succeeded) {
       mock_server_replies (
@@ -732,7 +732,7 @@ _test_heartbeat_events (bool pooled, bool succeeded)
 
    /* pooled client opens new socket, handshakes it by calling hello again */
    if (pooled && succeeded) {
-      request = mock_server_receives_legacy_hello (server, NULL);
+      request = mock_server_receives_any_hello (server);
       mock_server_replies (
          request,
          MONGOC_REPLY_NONE,
@@ -948,7 +948,7 @@ test_no_duplicates (void)
    client = mongoc_client_pool_pop (pool);
 
    /* Topology scanning thread starts, and sends a hello. */
-   request = mock_server_receives_legacy_hello (server, NULL);
+   request = mock_server_receives_any_hello (server);
    mock_server_replies_simple (request,
                                tmp_str ("{'ok': 1.0,"
                                         " 'isWritablePrimary': true, "
@@ -966,7 +966,7 @@ test_no_duplicates (void)
                                           NULL /* read prefs */,
                                           NULL /* reply */,
                                           &error);
-   request = mock_server_receives_legacy_hello (server, NULL);
+   request = mock_server_receives_any_hello (server);
    mock_server_replies_simple (
       request,
       tmp_str (
@@ -1055,7 +1055,8 @@ test_sdam_monitoring_install (TestSuite *suite)
       test_heartbeat_fails_dns_pooled,
       NULL,
       NULL,
-      test_framework_skip_if_offline);
+      test_framework_skip_if_offline,
+      test_framework_skip_if_rhel8_zseries);
    TestSuite_AddMockServerTest (
       suite,
       "/server_discovery_and_monitoring/monitoring/no_duplicates",
